// C3dTank.cpp
//

#include "stdafx.h"
#include "Stage.h"
#include "3dtank.h"
#include "math.h"

IMPLEMENT_DYNCREATE(C3dTank, C3dFrame) // for runtime class

C3dTank::C3dTank()
{
	// Create the frame
	C3dFrame::Create(NULL);

	// Load tank parts and build tank
	m_hull.Load(IDX_HULL);
	AddChild(&m_hull);
	m_turret.Load(IDX_TURRET);
	m_hull.AddChild(&m_turret);
	m_gun.Load(IDX_GUN);
	m_turret.AddChild(&m_gun);

	// The radar has its own frame for better control
	// of the rotation axis
	C3dFrame rframe;
	rframe.Create(&m_turret);
	C3dShape radar;
	radar.Load(IDX_RADAR);
	rframe.AddChild(&radar);
	radar.SetPosition(0, 0, -0.3);
	rframe.SetPosition(0, 0, 0.3);
	rframe.SetRotation(0, 1, 0, 0.1);

	SetGun(25);
}

C3dTank::~C3dTank()
{
}

#define D2R  0.01745329251994

void C3dTank::SetTurret(double angle)
{
	if ((angle < 0) || (angle >= 360)) {
		angle = 0;
	}
	double x = sin(angle * D2R);
	double z = cos(angle * D2R);
	m_turret.SetDirection(x, 0, z, &m_hull);
}

void C3dTank::SetGun(double angle)
{
	if (angle < 0) {
		angle = 0;
	} else if (angle >= 60) {
		angle = 60;
	}
	double y = -sin(angle * D2R);
	double z = cos(angle * D2R);
	m_gun.SetDirection(0, y, z, &m_turret);
}

void C3dTank::FireGun()
{
	PlaySound(MAKEINTRESOURCE(IDS_BANG),
		      AfxGetResourceHandle(),
			  SND_RESOURCE);

}



