// TankCtrl.cpp
//

#include "stdafx.h"
#include "Stage.h"
#include "3dtank.h"
#include "TankCtrl.h"

CTankCtrl::CTankCtrl()
{
	m_dGunAngle = 25;
	m_bWasFire = FALSE;
}

// virtual
void CTankCtrl::OnUpdate(_3DINPUTSTATE& st,
						 C3dFrame* pFrame)
{
	// Set the velocity (use y-axis for this)
	double v = st.dY / 2;

    // Get current position
	C3dVector pos;
	pFrame->GetPosition(pos);

	// Get current direction vector
    C3dVector dir, up;
    pFrame->GetDirection(dir, up);

	// Set new direction.
	// Both x and r affect direction.
	double dr = -st.dX + -st.dR;
	C3dMatrix r;
	r.Rotate(0, dr * 3, 0);
	dir = r * dir;

	// Multiply direction vector by velocity to get
	// position delta
	C3dVector ds = dir * v;

	// Set new position	and direction
	pos += ds;
	pFrame->SetPosition(pos);
	pFrame->SetDirection(dir);

	// Use POV info to set turret direction.
	// To do this we must be controlling a C3dTank,
	// not just a C3dFrame.
	C3dTank* pTank = (C3dTank*) pFrame;
	ASSERT(pTank->IsKindOf(RUNTIME_CLASS(C3dTank)));

	if (st.dPov >= 0) {
		pTank->SetTurret(st.dPov);
	}

	// Use buttons 3 and 4 to raise or lower the gun
	if (st.dwButtons & 0x04) {
		m_dGunAngle += 0.1;
	}
	if (st.dwButtons & 0x08) {
		m_dGunAngle -= 0.1;
	}
	if (m_dGunAngle < 0) {
		m_dGunAngle = 0;
	} else if (m_dGunAngle > 45) {
		m_dGunAngle = 45;
	}
	pTank->SetGun(m_dGunAngle);

	// Check whether it's time to fire
	if (st.dwButtons & 0x01) {
		if (!m_bWasFire) {
			pTank->FireGun();
			m_bWasFire = TRUE;
		}
	} else {
		m_bWasFire = FALSE;
	}
}

